@@ -120,7 +120,7 @@ def git_dir() -> str:
     return git('rev-parse', '--git-dir').strip('\n')
 
 def git_head() -> str:
-    return git('rev-parse', '--symbolic-full-name', 'HEAD').strip('\n')
+    return git('rev-parse', '-q', '--verify', '--symbolic-full-name', 'HEAD', _ok_code=[0, 1]).strip('\n')
 
 def find_remote_branches(name: str) -> List[Tuple[str, str]]:
     try:
@@ -164,13 +164,27 @@ def create_commit(branch: str, message: str, index: Path = None) -> str:
 
     return commit
 
-def commit_lfs_file(io: IO[bytes], branch: str, template: str = None):
+def parse_diff_entry(entry: str) -> Mapping[str, str]:
+    treediff, names = entry.split('\t', maxsplit=1)
+    srcmode, dstmode, srchash, dsthash, status = treediff.lstrip(':').split(' ')
+    srcname, _, dstname = names.partition('\t')
+    return {
+        'srcmode': srcmode,
+        'dstmode': dstmode,
+        'srchash': srchash,
+        'dsthash': dsthash,
+        'srcname': srcname,
+        'dstname': dstname,
+        'status': status,
+    }
+
+def commit_lfs_file(io: IO[bytes], branch: str, template: str = None, overwrite: bool = False):
     """
     Store the file in the LFS storage and commit it to a branch.
     """
     commit_lfs_files([io], branch)
 
-def commit_lfs_files(ios: Sequence[IO[bytes]], branch: str, template: str = None):
+def commit_lfs_files(ios: Sequence[IO[bytes]], branch: str, template: str = None, overwrite: bool = False):
     """
     Store the files in the LFS storage and commit them to a branch.
     """
@@ -178,7 +192,7 @@ def commit_lfs_files(ios: Sequence[IO[bytes]], branch: str, template: str = None
     hook_path = Path(git_dir()) / 'hooks' / 'pre-push'
     if not hook_path.is_file():
         try:
-            hook_path.parent.mkdir()
+            hook_path.parent.mkdir(exist_ok=True)
             hook_path.write_text(pre_push_hook)
             hook_path.chmod(0o755)
         except IOError as e:
@@ -205,6 +219,10 @@ def commit_lfs_files(ios: Sequence[IO[bytes]], branch: str, template: str = None
             if not diff:
                 logging.info(_("Nothing to commit"))
                 return
+            parsed_diff = [parse_diff_entry(d) for d in diff]
+            overwritten = [d['srcname'] for d in parsed_diff if d['srchash'] != ('0' * 40) and d['srcname'] != '.gitattributes']
+            if any(overwritten) and not overwrite:
+                raise Abort(_('would overwrite files: %s') % ', '.join(overwritten))
 
         if not template:
             template = "pristine-lfs data for %s"
